/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Optimoz Tweaks.
 *
 * The Initial Developer of the Original Code is Jens Bannmann.
 * Portions created by the Initial Developer are Copyright (C) 2003
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s) (alphabetical order):
 *  Jens Bannmann <jens.b@web.de>
 *  Piercarlo Slavazza <piercarlos@gmail.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var mtPrefSidebarSensitivity;
var mtPrefSidebarDelay;
var mtPrefSidebarHideMethod;
var mtPrefSidebarMaximizedOnly;

var mtAutohideLoaded;

var mtLeavingSidebar = false;
var mtWindowHasFocus = true;
var mtMouseDown = false;
var mtSidebarTimeout = null;
var mtMousePos = "";
var mtMouseSaved = "";
var mtSidebarBox = null;
var mtSidebarLocked = false;
var mtToggleButton = null;

// True after the sidebar was toggled by the mouse cursor staying in the
// trigger area, false when the cursor was moved out of that area again
var mtCursorStillInTriggerZone = false;

var mtSBWrapper_Firefox = {
  lastSidebar: "",

  init: function() {
    this.lastSidebar = mtGetPref("sidebar.last", "Char", "viewBookmarksSidebar");
  },

  isVisible: function() {
    return !mtSidebarBox.getAttribute("hidden");
  },

  toggle: function() {
    if (!this.isVisible()) {
      toggleSidebar(this.lastSidebar);
    } else {
      this.lastSidebar = mtSidebarBox.getAttribute("sidebarcommand");
  		mtPrefBranch.setCharPref("sidebar.last", this.lastSidebar);
  		toggleSidebar();
    }
  },
  
  installToggleButton: function(node) {
    node.setAttribute("app", "firefox");
    var sidebarTitle = document.getElementById("sidebar-title");
    sidebarTitle.parentNode.insertBefore(node, sidebarTitle);
  },
  
  uninstallToggleButton: function(node) {
    node.parentNode.removeChild(node);
  }
}
var mtSBWrapper_Seamonkey = {
  init: function() {},

  isVisible: function() {
    return !sidebar_is_hidden();
  },

  toggle: function() {
		SidebarShowHide();
  },
  
  installToggleButton: function(node) {
    node.setAttribute("app", "seamonkey");
    var titlebox = document.getElementById("sidebar-title-box");
    titlebox.insertBefore(node, titlebox.firstChild);
    titlebox.setAttribute("align", "center");
  },
  
  uninstallToggleButton: function(node) {
    var titlebox = document.getElementById("sidebar-title-box");
    titlebox.removeChild(node);
    titlebox.removeAttribute("align");
  }
}

var mtSidebar = mtIsFirefox ? mtSBWrapper_Firefox : mtSBWrapper_Seamonkey;

function mtInitSidebar() {
  mtPrefSidebarSensitivity = mtGetPref("sidebar.sensitivity", "Int", 3, 0, 50);
  mtPrefSidebarDelay = mtGetPref("sidebar.delay", "Int", 300, 0, 5000);
  mtPrefSidebarHideMethod = mtGetPref("sidebar.hidemethod", "Int", 0, 0, 1);
  mtPrefSidebarMaximizedOnly = mtGetPref("sidebar.maximizedonly", "Bool", false);

  mtSidebarBox = document.getElementById("sidebar-box");
  mtSidebar.init();

  var enabled = mtGetPref("sidebar.enabled", "Bool", true);
  if (enabled && !mtAutohideLoaded) {
    mtAutohideLoaded = true;
    mtSidebarStartup();
  } else if (!enabled && mtAutohideLoaded) {
    mtAutohideLoaded = false;
    mtSidebarShutdown();
  }
  mtManageToggleButton();
}

function mtSidebarStartup() {
  getBrowser().addEventListener("mousemove", mtMouseMoveListener, false);
  mtSidebarBox.addEventListener("mouseover", mtPrepareHideSidebar, false);
  mtSidebarBox.addEventListener("mousemove", mtSidebarMouseMoveListener, false);
  document.getElementById("sidebar-splitter").addEventListener("click", mtPreventHiding, true);
  document.getElementById("navigator-toolbox").addEventListener("mouseover", mtPreventHiding, true);
  window.addEventListener("mousedown", mtMouseStateListener, true);
  window.addEventListener("mouseup", mtMouseStateListener, true);
  window.addEventListener("blur", mtMouseStateListener, true);
  window.addEventListener("focus", mtMouseStateListener, true);
}

function mtSidebarShutdown() {
  getBrowser().removeEventListener("mousemove", mtMouseMoveListener, false);
  mtSidebarBox.removeEventListener("mouseover", mtPrepareHideSidebar, false);
  mtSidebarBox.removeEventListener("mousemove", mtSidebarMouseMoveListener, false);
  document.getElementById("sidebar-splitter").removeEventListener("click", mtPreventHiding, true);
  document.getElementById("navigator-toolbox").removeEventListener("mouseover", mtPreventHiding, true);
  window.removeEventListener("mousedown", mtMouseStateListener, true);
  window.removeEventListener("mouseup", mtMouseStateListener, true);
  window.removeEventListener("blur", mtMouseStateListener, true);
  window.removeEventListener("focus", mtMouseStateListener, true);
}

function mtManageToggleButton() {
  if (mtToggleButton == null && mtAutohideLoaded && mtPrefSidebarHideMethod == 0) {
    mtToggleButton = document.createElement("toolbarbutton");
    mtToggleButton.type = "checkbox";
    mtToggleButton.id = "toggleSidebarAutohide";
    mtToggleButton.tooltipText = mtBundle.getString("sidebarAutohide.toggleButton");
    mtSidebar.installToggleButton(mtToggleButton);
    mtToggleButton.addEventListener("command", mtToggleAutohide, false);
    mtSidebarLocked = false;
  } else if (mtToggleButton != null
             && (!mtAutohideLoaded || mtPrefSidebarHideMethod != 0)) {
    mtSidebar.uninstallToggleButton(mtToggleButton);
    mtSidebarLocked = false;
    mtToggleButton = null;
  }
}

function mtMouseMoveListener(e) {
  mtMousePos = e.screenX + "," + e.screenY;
  if (mtWindowHasFocus && document.getElementById("print-preview-toolbar") == null) {
    var xFrontier = mtPrefSidebarSensitivity + document.documentElement.boxObject.screenX;
    if (mtLeavingSidebar) {
      if (mtSidebar.isVisible()
          && !mtSidebarLocked
          && mtPrefSidebarHideMethod == 0
          && mtCheckMaximized()) {
        mtSidebar.toggle();
      }
      mtLeavingSidebar = false;
    } else if (e.screenX <= xFrontier
               && e.screenY > (100 + mtSidebarBox.boxObject.screenY)
               && !mtSidebar.isVisible()
               && !mtCursorStillInTriggerZone
               && mtCheckMaximized()) {
      window.clearTimeout(mtSidebarTimeout);
      mtSidebarTimeout = window.setTimeout(mtDelayedShowSidebar, mtPrefSidebarDelay);
      mtMouseSaved = mtMousePos;
    }
    if (e.screenX > xFrontier) {
      mtLeavingSidebar = false;
      mtCursorStillInTriggerZone = false;
    }
  }
}

function mtCheckMaximized() {
  return !mtPrefSidebarMaximizedOnly || document.documentElement.getAttribute("sizemode") == "maximized";
}

function mtSidebarMouseMoveListener(e) {
  mtMousePos = "#" + e.screenX + "," + e.screenY;
  var xFrontier = mtPrefSidebarSensitivity + document.documentElement.boxObject.screenX;
  if (mtPrefSidebarHideMethod == 1
      && e.screenX <= xFrontier
      && e.screenY > (100 + mtSidebarBox.boxObject.screenY)
      && !mtCursorStillInTriggerZone
      && mtCheckMaximized()) {
    window.clearTimeout(mtSidebarTimeout);
    mtSidebarTimeout = window.setTimeout(mtDelayedHideSidebar, mtPrefSidebarDelay);
    mtMouseSaved = mtMousePos;
  }
  if (e.screenX > xFrontier) {
    mtCursorStillInTriggerZone = false;
  }
}

function mtDelayedShowSidebar() {
  if (mtMouseSaved == mtMousePos && mtWindowHasFocus && !mtMouseDown) {
    mtSidebar.toggle();
    mtLeavingSidebar = true;
    mtCursorStillInTriggerZone = true;
  }
}

function mtDelayedHideSidebar() {
  if (mtMouseSaved == mtMousePos) {
    mtSidebar.toggle();
    mtCursorStillInTriggerZone = true;
  }
}

function mtPrepareHideSidebar(e) {
  if (mtWindowHasFocus && !mtMouseDown && !mtLeavingSidebar) {
    mtLeavingSidebar = true;
  }
}

function mtPreventHiding(e) {
  if (mtWindowHasFocus) {
    mtLeavingSidebar = false;
  }
}

function mtMouseStateListener(e) {
  switch(e.type) {
    case "mousedown": mtMouseDown = true; break;
    case "mouseup": mtMouseDown = false; break;
    case "blur": mtWindowHasFocus = false; break;
    case "focus": mtWindowHasFocus = true; break;
  }
}

function mtToggleAutohide(evt) {
  // Delay retrieval, as sometimes evt.target.checked seems to be reversed
	window.setTimeout("mtSidebarLocked = mtToggleButton.getAttribute('checked');", 0);
}
